#define REFLECTION
#define FRESNEL
#define SPECULAR

#ifdef GL_ES
varying mediump vec2 Texcoord;
varying mediump vec2 Env_Texcoord;
precision highp float;
#else
varying vec2 Texcoord;
varying vec2 Env_Texcoord;
//precision highp float;
#endif

varying vec3 vertex_pos;
varying vec3 Normal;

uniform float refl_scale;
uniform float switch_shining;
uniform float delta_time;
uniform float fSpecularPower;
uniform float reflectionScale;
uniform vec3 fvEyePosition;
uniform vec3 fvLightPosition;
uniform vec3 fvLightPosition2;
uniform vec4 fvAmbient;
uniform vec4 fvDiffuse;
uniform vec4 u_color;
uniform vec4 fvSpecular;
uniform sampler2D environmentMap;

//taken from somewhere in internet
float fresnel(float VdotN, float eta)
{
 float sqr_eta = eta * eta; // square of IOR
 float etaCos = eta * VdotN; // η·cos(Θ)
 float sqr_etaCos = etaCos*etaCos; // squared
 float one_minSqrEta = 1.0 - sqr_eta; // 1 – η2
 float value = etaCos - sqrt(one_minSqrEta + sqr_etaCos);
 value *= value / one_minSqrEta; // square and divide by 1 – η2
 return min(1.0, value * value); // square again
}

void main(void)
{  
   vec4  fvBaseColor      = texture2D( CC_Texture0, Texcoord );
   fvBaseColor.rgb /= fvBaseColor.a;
   vec3  fvLightDirection = normalize( fvLightPosition - vertex_pos );
   vec3  fvLightDirection2 = normalize( fvLightPosition2 - vertex_pos );
   vec3  fvNormal = normalize( Normal );
   //vec3  fvReflection     = normalize( ( ( 2.0 * fvNormal ) * dot( fvNormal, fvLightDirection ) ) - fvLightDirection ); 
   //vec3  fvReflection2     = normalize( ( ( 2.0 * fvNormal ) * dot( fvNormal, fvLightDirection2 ) ) - fvLightDirection2 ); 
   vec3  fvViewDirection  = normalize( fvEyePosition - vertex_pos );
      
   vec4  fvTotalAmbient   = fvAmbient * fvBaseColor; 
   
   float fNDotL           = max( 0.0, dot( fvNormal, fvLightDirection ));
   float fNDotL2           = max( 0.0, dot( fvNormal, fvLightDirection2 ));
   vec4  fvTotalDiffuse   = fvDiffuse * fNDotL * fvBaseColor; 
   vec4  fvTotalDiffuse2   = fvDiffuse * fNDotL2 * fvBaseColor;
   
   vec4  specular_term = vec4(0.0,0.0,0.0,0.0);
   vec4  specular_term2 = vec4(0.0,0.0,0.0,0.0);   
   
#ifdef SPECULAR
   //float fRDotV           = max( 0.0, dot( fvReflection, fvViewDirection ));
   //float fRDotV2           = max( 0.0, dot( fvReflection2, fvViewDirection ));
   float fRDotV  = clamp(dot(normalize(fvLightDirection  + fvViewDirection), fvNormal), 0.0, 1.0);
   float fRDotV2  = clamp(dot(normalize(fvLightDirection2  + fvViewDirection), fvNormal), 0.0, 1.0);
   
   specular_term    = fvSpecular * ( pow( fRDotV, fSpecularPower ) );
   specular_term2    = fvSpecular * ( pow( fRDotV2, fSpecularPower ) );
   
   //float avg_luminance = (fvBaseColor.r + fvBaseColor.g + fvBaseColor.b) * 0.33333;//length(fvBaseColor.rgb);
   
   //float diff = step(threshold , avg_luminance);
   specular_term = (specular_term * 10.0 )* fvBaseColor.a; //* avg_luminance;
   specular_term2 = (specular_term2 * 10.0 )* fvBaseColor.a; // * avg_luminance;
#endif
   
   float boost_shinning = (abs(sin(10.0 *( delta_time))) + 1.0) * (fvTotalAmbient.w + 1.0);
   fvTotalAmbient = (1.0 - switch_shining)*fvTotalAmbient + fvTotalAmbient * boost_shinning * switch_shining;
 
   vec4  reflcol = vec4(1.0,1.0,1.0,1.0);
   //vec4  reflcol2 = vec4(0.0,0.0,0.0,0.0);
   
#ifdef FRESNEL
   //float fVDotN = max( 0.0, dot( fvViewDirection, fvNormal ) );
   float fresnel_result = fresnel(fRDotV, 1.12);
   //float fresnel_result2 = fresnel(fVDotN, 0.99);
   vec4  reflcolor = vec4(1.0,1.0,1.0,1.0);
   reflcol = clamp(1.0-mix(fvTotalDiffuse, reflcolor, fresnel_result),0.0,1.0)*0.45;//*fvDiffuse.a;
   //reflcol2 = clamp(1.0-mix(fvTotalDiffuse, reflcolor, fresnel_result),0.0,1.0);
   //reflcol += reflcol2;
#endif

   vec4 refl = vec4(0.0,0.0,0.0,0.0);
#ifdef REFLECTION
   // Look up the reflection
   vec3 reflVec = reflect(-fvViewDirection, fvNormal);
   vec4 reflection = texture2D( environmentMap, reflVec.zx); //+ Env_Texcoord);//textureCube(Environment, reflVec.xyz);
   float cosine = dot(fvViewDirection, fvNormal);
   float sine = sqrt(1.0 - cosine * cosine);
   refl = sine * refl_scale * reflection * fvBaseColor.a ;
#endif
   fvTotalDiffuse += fvTotalDiffuse2;
   specular_term += specular_term2;
   //gl_FragColor = u_color * (reflcol);//   + sine * refl   
   gl_FragColor = u_color * (fvTotalAmbient  + fvTotalDiffuse* reflcol + specular_term  + refl);  
   gl_FragColor.w = switch_shining * fvAmbient.a + (1.0 - switch_shining) * 1.0;
}
