uniform float sampleDist;
uniform float sampleStrength;

varying vec2 v_texCoord;
uniform sampler2D CurFrame;

void main()
{
	float samples[3];
	samples[0] = 0.08;
	samples[1] = 0.05;
	samples[2] = 0.03;
	//samples[3] = -0.02;
	//samples[4] = -0.01;
	
	//Vector from pixel to the center of the screen
	vec2 dir = 0.5 - v_texCoord;
	
	//Distance from pixel to the center (distant pixels have stronger effect)
	//float dist = distance( vec2( 0.5, 0.5 ), texCoord );
	//float dist = sqrt( dir.x*dir.x + dir.y*dir.y );
	float dist = dir.x*dir.x + dir.y*dir.y;
	
	//Now that we have dist, we can normlize vector
	//dir = normalize( dir );
	
	//Save the color to be used later
	vec4 color = texture2D( CurFrame, v_texCoord );
	//Average the pixels going along the vector
	vec4 sum = color;

	vec4 res=texture2D( CurFrame, v_texCoord + dir * samples[0] * sampleDist );
	sum += res;
	res=texture2D( CurFrame, v_texCoord + dir * samples[1] * sampleDist );
	sum += res;
	res=texture2D( CurFrame, v_texCoord + dir * samples[2] * sampleDist );
	sum += res;
	
	sum /= 4.0;
	
	//Calculate amount of blur based on
	//distance and a strength parameter
	float t = dist * sampleStrength;
	t = clamp( t, 0.0, 1.0 );//We need 0 <= t <= 1
	
	//Blend the original color with the averaged pixels
	gl_FragColor = mix( color, sum, t );
}